import csv
import dateutil
from cStringIO import StringIO

from Acquisition import aq_inner
from Products.Five.browser import BrowserView
from Products.CMFCore.utils import getToolByName

from plone.app.textfield.interfaces import ITransformer

from Products.Poi.adapters import IResponseContainer


class CSVExport(BrowserView):
    """ view to produce the CSV exports
    """

    def __call__(self):
        context = aq_inner(self.context)
        encoding = self.request.get('encoding')
        issuefolder = context.restrictedTraverse('@@issuefolder-sunpower')
        pas_member = context.restrictedTraverse('@@pas_member')

        issues = issuefolder.getFilteredIssues(self.request)
        buffer = StringIO()

        writer = csv.writer(buffer)
        header = [
            '#',
            'Title',
            'Tags',
            'State',
            'Submitted by',
            'Submitted date/time',
            'Contact Email',
            'Responsible Manager',
            'Delivery #',
            'Load ID',
            'Origin Name',
            'Origin City',
            'Origin State',
            'Origin Country',
            'Destination Name',
            'Destination City',
            'Destination State',
            'Category',
            'Sub-Category',
            'Claim Status',
            'Why - 1',
            'Why - 2',
            'Why - 3',
            'Preventative Measure',
            'Preventative Measure Due Date',
            'Response Due Date',
            'Last modified by',
            'Last modified by date/time',
            'Details',
            'Software Engineer Time',
            'Business Analyst Time',
            'TranSend/EDI Support Time',
            'Carrier'
        ]
        writer.writerow(header)
        # to get the previous person who changed something
        # we need to get issue responses
        mt = getToolByName(self.context, 'portal_membership')
        for issue in issues:
            obj = issue.getObject()
            responsefolder = IResponseContainer(obj)
            responses = []
            for id, response in enumerate(responsefolder):
                if response is None:
                    # Has been removed.
                    continue
                responses.append({'creator': response.creator,
                                 'date': response.date})
            # the responses are in order so we just grab the last one
            if len(responses) > 0:
                last_actor = responses[-1]['creator']
                last_modified = responses[-1]['date']
            else:
                last_actor = obj.Creator()
                last_modified = obj.modified()
            actor_info = mt.getMemberInfo(last_actor)
            if actor_info and actor_info.get("fullname", None):
                last_actor = actor_info["fullname"]
            row = []
            row.append(issue.id)
            row.append(issue.Title)
            row.append(", ".join(sorted((y for y in issue.Subject),
                                        key=lambda x: x.lower())))
            row.append(obj.getReviewState()['title'].encode('utf-8'))
            row.append(
                pas_member.info(issue.Creator)['name_or_id'].encode('utf-8'))
            row.append(dateutil.parser.parse(
                issue.CreationDate).strftime('%Y-%m-%d %H:%M:%S'))

            assignee = ""
            if obj.assignee:
                member = mt.getMemberInfo(obj.assignee)
                if member is not None:
                    assignee = member.get("fullname", None)
                else:
                    assignee = obj.assignee

            type_of_issue = issue.type_of_issue \
                and obj.display_option('possibleTypesOfIssues') or ""

            sub_category = obj.sub_category \
                and obj.display_option('possibleSubCategories') or ""

            claim_status = obj.claim_status \
                and obj.display_option('possibleClaimStatuses') or ""

            why1 = obj.why1 \
                and obj.display_option('possibleWhy1Options') or ""

            why2 = obj.why2 \
                and obj.display_option('possibleWhy2Options') or ""

            if issue.destination_name \
                    and type(issue.destination_name) is tuple:
                destination_name = ", ".join(issue.destination_name)
            else:
                destination_name = issue.destination_name

            cols = [
                obj.contact_email,
                assignee,
                obj.delivery_number,
                obj.load_id,
                obj.origin_name,
                obj.origin_city,
                obj.origin_state,
                obj.origin_country,
                destination_name,
                obj.destination_city,
                obj.destination_state,
                type_of_issue,
                sub_category,
                claim_status,
                why1,
                why2,
                obj.why3,
            ]

            for i in cols:
                row.append(self.prepValue(i))

            transformer = ITransformer(obj)

            row.append(obj.preventative_measure and
                       obj.preventative_measure.encode('utf-8'))

            row.append(obj.preventative_due and
                       obj.preventative_due.strftime('%Y-%m-%d'))

            row.append(obj.requested_by and
                       obj.requested_by.strftime('%Y-%m-%d'))
            row.append(last_actor.encode('utf-8'))
            row.append(last_modified.strftime('%Y-%m-%d %H:%M:%S'))

            if obj.details:
                transformedDetails = transformer(obj.details, 'text/plain')

            row.append(transformedDetails and
                       transformedDetails.encode('utf-8') or "")

            row.append(obj.software_engineer_time_total and
                       str(obj.software_engineer_time_total) or "")
            row.append(obj.business_analyst_time_total and
                       str(obj.business_analyst_time_total) or "")
            row.append(obj.transend_edi_support_time_total and
                       str(obj.transend_edi_support_time_total) or "")

            row.append(obj.carrier and obj.display_option("possibleCarriers") or "")

            writer.writerow(row)
        value = buffer.getvalue()
        value = unicode(value, "utf-8").encode("iso-8859-1", "replace")

        if not encoding:
            encoding = 'UTF-8'
        self.request.response.setHeader('Content-type',
                                        'text/csv;charset='+encoding)
        self.request.response.setHeader('Content-Disposition',
                                        'attachment; filename=export.csv')

        return value

    def prepValue(self, value):
        if value != None:
            return value.encode('utf-8')
        else:
            return ""
