import json

from Acquisition import aq_inner
from plone import api

from Products.PythonScripts.standard import url_quote

from Products.Poi.content.issue import IIssue
from Products.Poi.browser.tracker import IssueFolderView as IssueFolderViewOrig
from Products.Poi.browser.tracker import QuickSearchView as QuickSearchViewOrig

from retrans.poi.browser.tracker \
    import TrackerJsonInfoView as TrackerJsonInfoViewOrig
from sunpower.poi.content.sunpowertracker import possibleTypesOfIssues


class IssueFolderView(IssueFolderViewOrig):
    """ customization of tracker view for SunPower """

    def buildIssueSearchQuery(self, criteria=None, **kwargs):
        """Build canonical query for issue search.
        """
        context = aq_inner(self.context)

        if criteria is None:
            criteria = kwargs
        else:
            criteria = dict(criteria)

        allowedCriteria = {
            'type_of_issue': 'type_of_issue',
            'state': 'review_state',
            'tags': 'Subject',
            'assignee': 'assignee',
            'text': 'SearchableText',
            'id': 'getId',
            'created': 'created',
            'modified': 'modified',
            'requested_by': 'requested_by'
        }

        query = {}
        query['path'] = query['path'] = '/'.join(context.getPhysicalPath())
        query['object_provides'] = IIssue

        for k, v in allowedCriteria.items():
            if criteria.get(k):
                if criteria[k] == '(UNASSIGNED)':
                    continue
                query[v] = criteria[k]
            elif criteria.get(v):
                query[v] = criteria[v]

        # Playing nicely with the form.

        # Subject can be a string of one tag, a tuple of several tags
        # or a dict with a required query and an optional operator
        # 'and/or'.  We must avoid the case of the dict with only the
        # operator and no actual query, else we will suffer from
        # KeyErrors.  Actually, when coming from the
        # poi_issue_search_form, instead of say from a test, its type
        # is not 'dict', but 'instance', even though it looks like a
        # dict.  See http://plone.org/products/poi/issues/137
        if 'Subject' in query:
            subject = query['Subject']
            # We cannot use "subject.has_key('operator')" or
            # "'operator' in subject'" because of the strange
            # instance.
            try:
                subject['operator']
            except TypeError:
                # Fine: subject is a string or tuple.
                pass
            except KeyError:
                # No operator, so nothing can go wrong.
                pass
            else:
                try:
                    subject['query']
                except KeyError:
                    del query['Subject']

        query['sort_on'] = criteria.get('sort_on', 'created')
        query['sort_order'] = criteria.get('sort_order', 'reverse')
        if criteria.get('sort_limit'):
            query['sort_limit'] = criteria.get('sort_limit')
        # allow substring searches
        if 'SearchableText' in query:
            query['SearchableText'] = '*{}*'.format(query['SearchableText'])

        # handle the date range searches
        for key in ['created', 'modified', 'requested_by']:
            if query.get(key) and query[key].get('query') is None:
                del query[key]

        return query

    def getCategory(self):
        return possibleTypesOfIssues(aq_inner(self.context))


class QuickSearchView(QuickSearchViewOrig):
    """Parse a quicksearch string and jump to the appropriate issue or
    search result page.

    """

    def __call__(self):
        tracker = aq_inner(self.context)
        search_text = self.request.form.get('searchText', '')
        issue_id = search_text
        if issue_id.startswith('#'):
            issue_id = issue_id[1:]
        base_url = tracker.absolute_url()
        if issue_id in tracker.keys():
            url = '%s/%s' % (base_url, issue_id)
        else:
            url = '%s/sunpower_poi_issue_search?SearchableText=%s' % (
                base_url, url_quote(search_text))
        self.request.RESPONSE.redirect(url)


class TrackerJsonInfoView(IssueFolderView):

    def __call__(self, **kwargs):

        context = aq_inner(self.context)
        info = {}

        open_states = [
            'open',
            'in-progress',
            'unconfirmed',
            'fixed',
            'root-cause-analysis',
            'closure-review'
        ]

        # # get the areas totals
        # info['areas'] = []
        # for area in context.available_areas:
        #     info['areas'].append({
        #             "key": area['short_name'],
        #             "total": len(self.getFilteredIssues(
        #                     area=area['short_name'],
        #                     state=open_states
        #                 ))
        #         })

        # # get the target release totals
        # if context.isUsingReleases():
        #     info['releases'] = []
        #     for release in context.available_releases:
        #         info['releases'].append({
        #                 "key": release[0],
        #                 "total": len(self.getFilteredIssues(
        #                         targetRelease=release[0],
        #                         state=open_states
        #                     ))
        #             })


        # get the category totals
        info['category'] = []
        for category in self.getCategory():
            info['category'].append({
                    "key": category.value,
                    "total": len(self.getFilteredIssues(
                            type_of_issue=category.value
                        )
                    )
                })


        # get the state totals
        info['states'] = []
        info['states'].append({
                "key": "all",
                "total":len(self.getFilteredIssues(state=open_states))
            })
        for state in context.getIssueWorkflowStates():
            info['states'].append({
                    "key": state[0],
                    "total": len(self.getFilteredIssues(state=state[0]))
                })


        # # get the tag totals
        # member = api.user.get_current()
        # isManager = member.has_role('Manager', context)
        # tags = context.getTagsInUse()
        # if isManager and len(tags) > 0:
        #     info['tags'] = []
        #     for tag in tags:
        #         info['tags'].append({
        #                 "key": tag[0],
        #                 "total": tag[1]
        #             })


        response = self.request.response
        response.setHeader('Content-Type', 'application/json; charset=utf-8')
        response.setBody(json.dumps(info))
        return response