# -*- coding: utf-8 -*-
"""Setup tests for this package."""
from sunpower.poi.testing import SUNPOWER_POI_INTEGRATION_TESTING  # noqa
from plone import api

import unittest


class TestSetup(unittest.TestCase):
    """Test that sunpower.poi is properly installed."""

    layer = SUNPOWER_POI_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')

    def test_product_installed(self):
        """Test if sunpower.poi is installed."""
        self.assertTrue(self.installer.isProductInstalled(
            'sunpower.poi'))

    def test_browserlayer(self):
        """Test that ISunPowerPoiLayer is registered."""
        from sunpower.poi.interfaces import (
            ISunPowerPoiLayer)
        from plone.browserlayer import utils
        self.assertIn(ISunPowerPoiLayer, utils.registered_layers())


class TestUninstall(unittest.TestCase):

    layer = SUNPOWER_POI_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer['portal']
        self.installer = api.portal.get_tool('portal_quickinstaller')
        self.installer.uninstallProducts(['sunpower.poi'])

    def test_product_uninstalled(self):
        """Test if sunpower.poi is cleanly uninstalled."""
        self.assertFalse(self.installer.isProductInstalled(
            'sunpower.poi'))

    def test_browserlayer_removed(self):
        """Test that ISunPowerPoiLayer is removed."""
        from sunpower.poi.interfaces import ISunPowerPoiLayer
        from plone.browserlayer import utils
        self.assertNotIn(ISunPowerPoiLayer, utils.registered_layers())
