import logging

from plone import api

from Products.CMFCore.utils import getToolByName
from Products.CMFCore.WorkflowCore import WorkflowException

from sixfeetup.utils import helpers

from sunpower.poi.content.sunpowerissue import SunPowerIssue

logger = logging.getLogger('sunpower.poi.upgrades')


def migrate_custom_types(context):
    """Migrate SunPowerIssue class from retrans.poi to sunpower.poi."""
    portal = api.portal.get()
    context = portal['sunpower_old']
    target = portal['sunpower']
    uid = target.UID()
    issues = api.content.find(context=context, portal_type='SunPowerIssue')
    wtool = getToolByName(portal, 'portal_workflow')
    mt = getToolByName(portal, 'portal_membership')
    issue_histories = {}

    for issue in issues:
        obj = issue.getObject()
        api.content.move(source=obj, target=target)
        obj._tracker_uid = uid

        # retrofit the class of the issue to be based in sunpower.poi
        # instead of retrans.poi
        obj.__class__ = SunPowerIssue

        # compile the review histories to restore after changing workflow
        with api.env.adopt_roles(['Editor', 'Member']):
            review_history = wtool.getInfoFor(obj, 'review_history')
        issue_histories[obj.id] = review_history

    # change the workflow for SunPowerIssue
    wtool.setChainForPortalTypes(
        ('SunPowerIssue',), 'poi_sunpowerissue_workflow')

    # loop back over the issues and replay the review_history
    for issue in issues:
        obj = issue.getObject()
        obj.reindexObjectSecurity()
        review_history = issue_histories[obj.id]

        for change in review_history:
            action = change['action']
            if action is None:
                continue
            actor = change['actor']
            if not mt.getMemberById(actor):
                actor = 'admin'
            cur_state = api.content.get_state(obj=obj)
            with api.env.adopt_user(username=actor) and \
                    api.env.adopt_roles(['Technician']):
                try:
                    wtool.doActionFor(obj, action)
                except WorkflowException:
                    logger.info("Could not transition: " + obj.id + ": " +
                                action + ", " + actor + ", " + cur_state)
                    pass
                else:
                    logger.info("Action complete: " + obj.id + ": " + action +
                                ", " + actor + ", " + cur_state)
